Struct sp_tracing::event::Event [−][src]
pub struct Event<'a> { /* fields omitted */ }Expand description
Events represent single points in time where something occurred during the
execution of a program.
An Event can be compared to a log record in unstructured logging, but with
two key differences:
Events exist within the context of a span. Unlike log lines, they may be located within the trace tree, allowing visibility into the temporal context in which the event occurred, as well as the source code location.- Like spans,
Events have structured key-value data known as fields, which may include textual message. In general, a majority of the data associated with an event should be in the event’s fields rather than in the textual message, as the fields are more structured.
Implementations
Constructs a new Event with the specified metadata and set of values,
and observes it with the current subscriber.
Returns a new Event in the current span, with the specified metadata
and set of values.
Returns a new Event as a child of the specified span, with the
provided metadata and set of values.
Constructs a new Event with the specified metadata and set of values,
and observes it with the current subscriber and an explicit parent.
Visits all the fields on this Event with the specified visitor.
Returns true if the new event’s parent should be determined based on the current context.
If this is true and the current thread is currently inside a span, then that span should be the new event’s parent. Otherwise, if the current thread is not inside a span, then the new event will be the root of its own trace tree.